
/*
 * Copyright (c) 2004 Martin Cernohorsky
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the developer may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPER ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE DEVELOPER BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termcap.h>

#include "scr.h"

static struct {
	/* Cursor movement */
	char *cl; /* clear screen */
	char *cm; /* cursor move */
	char *ho; /* home */
	char *bl; /* bell */
	char *cd; /* clear to end of display */
	char *ce; /* clear to EOL */
	char *dc; /* delete char */
	char *dl; /* delete line */
	char *ic; /* insert char */
	char *ll; /* last line */

	/* Attributes */
	char *us; /* underscore start */
	char *ue; /* underscore end */
	char *uc; /* underscore one char and move past it */
	char *mb; /* blink on */
	char *md; /* bold on */
	char *mh; /* half bright on */
	char *mr; /* reverse on */
	char *so; /* standout on */
	char *se; /* standout end */
	char *me; /* all attributes end */

	/* Misc */
	char *rs; /* reset tty completely to sane modes */
	char *sc; /* save cursor pos */
	char *rc; /* restore cursor pos */
	char *sf; /* scroll text forward (up) */
	char *sr; /* scroll text backward (down) */
	char *ve; /* cursor normal */
	char *vs; /* cursor invisible */
} caps;

static int first = 1;

int
term_init(char *name)
{
	static char term[1024];
	static char data[2048];
	char *pp = data;

	if (name == NULL) name = "vt100";
	if (name[0] == '\0') name = "vt100";

	if (tgetent(term, name) != 1) return (-1); /* no termcap entry */

	/* Cursor movement */
	caps.cl = tgetstr("cl", &pp);
	caps.cm = tgetstr("cm", &pp);
	caps.ho = tgetstr("ho", &pp);
	caps.bl = tgetstr("bl", &pp);
	caps.cd = tgetstr("cd", &pp);
	caps.ce = tgetstr("ce", &pp);
	caps.dc = tgetstr("dc", &pp);
	caps.dl = tgetstr("dl", &pp);
	caps.ic = tgetstr("ic", &pp);
	caps.ll = tgetstr("ll", &pp);

	/* Attributes */
	caps.us = tgetstr("us", &pp);
	caps.ue = tgetstr("ue", &pp);
	caps.uc = tgetstr("uc", &pp);
	caps.mb = tgetstr("mb", &pp);
	caps.md = tgetstr("md", &pp);
	caps.mh = tgetstr("mh", &pp);
	caps.mr = tgetstr("mr", &pp);
	caps.so = tgetstr("so", &pp);
	caps.se = tgetstr("se", &pp);
	caps.me = tgetstr("me", &pp);

	/* Misc */
	caps.rs = tgetstr("rs", &pp);
	caps.sc = tgetstr("sc", &pp);
	caps.rc = tgetstr("rc", &pp);
	caps.sf = tgetstr("sf", &pp);
	caps.sr = tgetstr("sr", &pp);
	caps.ve = tgetstr("ve", &pp);
	caps.vs = tgetstr("vs", &pp);

	/* now some sanity checks */
	if (caps.cm == NULL) return (-1); /* no addressable cursor */
	if (caps.ho == NULL) caps.ho = strdup(tgoto(caps.cm, 0, 0));
	if (caps.bl == NULL) caps.bl = "\007";
	if (caps.ll == NULL) caps.ll = strdup(tgoto(caps.cm, 1, 24));

	first = 0;

	return (0); /* all goes well */
}

char *
term_clear()
{
	return(caps.cl);
}

char *
term_goto(int col, int row)
{
	return(tgoto(caps.cm, col, row));
}

char *
term_home()
{
	return(caps.ho);
}

char *
term_lastline()
{
	return(caps.ll);
}

char *
term_bell()
{
	return(caps.bl);
}

char *
term_clear2eod()
{
	return(caps.cd);
}

char *
term_clear2eol()
{
	return(caps.ce);
}

char *
term_delchar()
{
	return(caps.dc);
}

char *
term_delline()
{
	return(caps.dl);
}

char *
term_inschar()
{
	return(caps.ic);
}

char *
term_attr(int attr)
{
	switch(attr)
	{
	case TERM_BLINK:
		return(caps.mb);
	case TERM_BOLD:
		return(caps.md);
	case TERM_HALF:
		return(caps.mh);
	case TERM_REVERSE:
		return(caps.mr);
	case TERM_ALL:
		return(caps.me);

	case TERM_STANDOUT:
		return(caps.so);
	case TERM_UNDERSCORE:
		return(caps.us);
	default:
		return("");
	}
}

char *
term_attr_off(int attr)
{
	switch (attr)
	{
	case TERM_STANDOUT:
		return(caps.se);
	case TERM_UNDERSCORE:
		return(caps.ue);
	default:
		return(caps.me);
	}
}

char *
term_reset()
{
	return(caps.rs);
}

char *
term_savepos()
{
	return(caps.sc);
}

char *
term_restorepos()
{
	return(caps.rc);
}

char *
term_scrollup()
{
	return(caps.sf);
}

char *
term_scrolldown()
{
	return(caps.sr);
}

char *
term_cursor_invis()
{
	return(caps.vs);
}

char *
term_cursor_vis()
{
	return(caps.ve);
}

